/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * © 2013 Open-Xchange Inc., Tarrytown, NY, USA. info@open-xchange.com
 *
 * @author Daniel Dickhaus <daniel.dickhaus@open-xchange.com>
 */
 
define('plugins/notifications/tutorial/register',
    ['io.ox/core/extensions',
     'less!plugins/notifications/tutorial/style.less'], function (ext) {

    'use strict';
    
    //simple helper to trigger some events
    //normally this is done by mailapi, taskapi, etc.
    var myEventTriggerer = {
            lookForItems: function () {
                //build some items and put them in an array
                var items = [{title: 'I am a notification', description: 'Hello world!'},
                             {title: 'I am a notification too', description: 'Hooray!'}];
                //trigger the event to add them
                $(myEventTriggerer).trigger('set-tutorial-notification', [items]);
            }
        };
    
    //register our notification plugin
    ext.point('io.ox/core/notifications/register').extend({
        id: 'tutorial',//unique id
        index: 500, //unused index
        register: function (controller) {
            //give our plugin a name and send it to the controller together with our view
            var notifications = controller.get('io.ox/tutorial', NotificationsView);
            
            //now some basic methods for reseting, removing and adding notifications
            //note: the controller listens to changes in the collection, to redraw it
            
            //fill our collection of notification models with new ones
            //items here is an array of Objects containing our attributes
            function reset(e, items) {
                var models = [];
                items = [].concat(items);//make sure we have an array
                _(items).each(function (item) {
                    models.push(new Backbone.Model(item));
                });
                notifications.collection.reset(models);//fill the collection
            }
            
            //now add the event listeners
            $(myEventTriggerer).on('set-tutorial-notification', reset);

            //just to make sure it gets filled with values on load we trigger our search method
            myEventTriggerer.lookForItems();
        }
    });
    
    //the view of our plugin
    var NotificationsView = Backbone.View.extend({

        className: 'notifications',
        id: 'io-ox-notifications-tutorial',
        
        //events from our items
        events: {
            'click .item': 'openPopup',
            'click [data-action="close"]': 'closeNotification'
        },

        //draws the plugin
        render: function () {
            //build baton to wrap things up
            var baton = ext.Baton({ view: this });
            //draw header and container by creating an extension point and invoke drawing on its extensions
            ext.point('io.ox/core/notifications/tutorial/header').invoke('draw', this.$el.empty(), baton);
            
            //loop over collection and draw
            this.collection.each(function (model) {
                baton = ext.Baton({ model: model, view: this });
                ext.point('io.ox/core/notifications/tutorial/item').invoke('draw', this.$('.notifications'), baton);//draw the item
            }, this);
            return this;
        },
        //the action for our sidepopup
        openPopup: function (e) {
            
            var overlay = $('#io-ox-notifications-overlay'),//the overlay we draw our sidepopup in
                cid = $(e.currentTarget).attr('data-cid'),//getting the right model
                model = this.collection.getByCid(cid);
            
            require(['io.ox/core/tk/dialogs'], function (dialogs) {//require dialogs
                //create the popup
                new dialogs.SidePopup({ arrow: false, side: 'right' })
                    .setTarget(overlay)
                    .show(e, function (popup) {
                        //fill it with our data
                        popup.append($('<div>').text(model.get('title')),
                                     $('<br>'),
                                     $('<div>').text(model.get('description')));
                    });
            });
        },
        //the action for the close button
        closeNotification: function (e) {
            e.stopPropagation();//to prevent sidepopup from opening
            
            var cid = $(e.currentTarget).closest('.item').attr('data-cid'),//getting the right model
                model = this.collection.getByCid(cid);
            
            this.collection.remove(model);//remove it from the collection
        }
    });
    
    //the header and container for the notification plugin
    ext.point('io.ox/core/notifications/tutorial/header').extend({
        draw: function (baton) {
            this.append(
                $('<legend class="section-title">').text('Hello World'),//header
                $('<div class="notifications">')//the container for our notifications
            );
        }
    });
    
    //draw a single notification item
    ext.point('io.ox/core/notifications/tutorial/item').extend({
        draw: function (baton) {
            this.append(
            $('<div class="tutorial item">')
            .attr('data-cid', baton.model.cid)//needed for identification
                .append(
                    $('<div class="mytext">').text(baton.model.get('title')),//some text to fill it
                    $('<button class="mybutton btn btn-primary" data-action="close">').text('close')//close button
                )
            );
        }
    });
    
    //just to give something back
    return true;
});